-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

-- Use the table provided for us
local NAME, RC = ...

-- Make our root addon table accessible via /run etc for easier debugging
RaidChecklist = RC

-- Pull in the localized strings
L = RaidChecklistStrings
P = RaidChecklistPetFamilies

RC.colorcodedbackdrop = true


-- The followings strings in the buffs and debuffs arrays are used as keys and should be localized
RC.buffs = {
  "CRIT", "ATTACKPWR", "SPELLPWR6", "SPELLPWR10",
  "PHYSHASTE", "SPELLHASTE",
  "STATS", "ALLDMG", "AGISTR", "STAM", "BLOODLUST",
  "MANA", "MANAREGEN", "REPLENISHMENT",
}

RC.debuffs = {
  "SPELLCRIT",
  "PHYSDMGTAKEN", "SPELLDMGTAKEN", "BLEEDDMGTAKEN",
  "ATTACKSPD", "CASTSPD", "DEMOROAR",
  "SUNDER", "MORTALSTRIKE",
}
-- Note: Improved Fairy Fire also has a +SpellHitChance debuff, but seems alone in providing that?

RC.ccpvps = {
  "DISORIENT", "SILENCE", "SNARE", "ROOT", "KNOCKBACK", "DISARM", "STUN"
}


-- These aren't localized
local SPEC = " " local NL = "\n "

local availablefrom = {
  STATS         =
                  L.PALADIN..NL..
                  L.DRUID..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.SHALESPIDER,
  AGISTR        =
                  L.WARRIOR..NL..
                  L.SHAMAN..NL..
                  L.DEATHKNIGHT..NL..
                  L.HUNTER..L.PET..P.CAT..NL..
                  L.HUNTER..L.PET..P.SPIRITBEAST,
  STAM          =
                  L.PRIEST..NL..
                  L.WARRIOR..NL..
                  L.WARLOCK..L.PET..P.IMP..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.SILITHID,
  ALLDMG        =
                  L.HUNTER..SPEC..L.BEASTMASTER..NL..
                  L.MAGE..SPEC..L.ARCANE..NL..
                  L.PALADIN..SPEC..L.RETRIBUTION,
  CRIT          =
                  L.DRUID..SPEC..L.FERAL..NL..
                  L.WARRIOR..SPEC..L.FURY..NL..
                  L.ROGUE..SPEC..L.SUBTLETY..NL..
                  L.SHAMAN..SPEC..L.ELEMENTAL..NL..
                  L.HUNTER..L.PET..P.WOLF..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.DEVILSAUR,
  ATTACKPWR     =
                  L.PALADIN..NL..
                  L.DEATHKNIGHT..NL..
                  L.HUNTER..SPEC..L.MARKSMAN..NL..
                  L.SHAMAN..SPEC..L.ENHANCEMENT,
  SPELLPWR6     =
                  L.MAGE..NL..
                  L.SHAMAN,
  SPELLPWR10    =
                  L.SHAMAN..SPEC..L.ELEMENTAL..NL..
                  L.WARLOCK..SPEC..L.DEMONOLOGY..NL..
                  L.MAGE..SPEC..L.FIRE,
  PHYSHASTE     =
                  L.SHAMAN..NL..
                  L.DEATHKNIGHT..SPEC..L.FROST..NL..
                  L.HUNTER..SPEC..L.SURVIVAL,
  SPELLHASTE    =
                  L.SHAMAN..NL..
                  L.DRUID..SPEC..L.BALANCE..NL..
                  L.PRIEST..SPEC..L.SHADOW,
  BLOODLUST     =
                  L.SHAMAN..NL..
                  L.MAGE..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.COREHOUND,
  MANA          =
                  L.MAGE..NL..
                  L.WARLOCK..L.PET..P.FELHUNTER,
  MANAREGEN     =
                  L.PALADIN..NL..
                  L.SHAMAN..NL..
                  L.WARLOCK..L.PET..P.FELHUNTER,
  REPLENISHMENT =
                  L.PRIEST..SPEC..L.SHADOW..NL..
                  L.MAGE..SPEC..L.FROST..NL..
                  L.PALADIN..SPEC..L.RETRIBUTION..NL..
                  L.DRUID..SPEC..L.RESTORATION..NL..
                  L.WARLOCK..SPEC..L.DESTRUCTION,

  SPELLCRIT     =
                  L.MAGE..SPEC..L.FIRE..NL..
                  L.WARLOCK..SPEC..L.DESTRUCTION,
  PHYSDMGTAKEN  =
                  L.WARRIOR..SPEC..L.ARMS..NL..
                  L.ROGUE..SPEC..L.COMBAT..NL..
                  L.DEATHKNIGHT..SPEC..L.FROST..NL..
                  L.HUNTER..L.PET..P.RAVAGER..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.WORM,
  SPELLDMGTAKEN =
                  L.WARLOCK..NL..
                  L.ROGUE..SPEC..L.ASSASSINATION..NL..
                  L.DEATHKNIGHT..SPEC..L.UNHOLY..NL..
                  L.DRUID..SPEC..L.BALANCE..NL..
                  L.HUNTER..L.PET..P.DRAGONHAWK..NL..
                  L.HUNTER..L.PET..P.WINDSERPENT,
  BLEEDDMGTAKEN =
                  L.DRUID..SPEC..L.FERAL..NL..
                  L.ROGUE..SPEC..L.SUBTLETY..NL..
                  L.WARRIOR..SPEC..L.ARMS..NL..
                  L.HUNTER..L.PET..P.HYENA..NL..
                  L.HUNTER..L.PET..P.BOAR..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.RHINO,
  ATTACKSPD     =
                  L.WARRIOR..NL..
                  L.DEATHKNIGHT..NL..
                  L.DRUID..SPEC..L.FERAL..NL..
                  L.PALADIN..SPEC..L.PROTECTION..NL..
                  L.HUNTER..L.PET..P.FOX..NL..
                  L.HUNTER..L.PET..P.TALLSTRIDER,
  CASTSPD       =
                  L.WARLOCK..NL..
                  L.ROGUE..NL..
                  L.MAGE..SPEC..L.ARCANE..NL..
                  L.HUNTER..L.PET..P.SPOREBAT..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.COREHOUND,
  DEMOROAR      =
                  L.WARRIOR..SPEC..L.PROTECTION..NL..
                  L.DRUID..SPEC..L.DRUIDBEAR..NL..
                  L.PALADIN..SPEC..L.PROTECTION..NL..
                  L.DEATHKNIGHT..SPEC..L.BLOOD..NL..
                  L.HUNTER..L.PET..P.BEAR,
  SUNDER        =
                  L.ROGUE..NL..
                  L.WARRIOR..SPEC..L.PROTECTION..NL..
                  L.DRUID..SPEC..L.DRUIDBEAR..NL..
                  L.HUNTER..L.PET..P.SERPENT..NL..
                  L.HUNTER..L.PET..P.RAPTOR,
  MORTALSTRIKE  =
                  L.ROGUE..NL..
                  L.HUNTER..NL..
                  L.WARRIOR..SPEC..L.ARMS..NL..
                  L.WARRIOR..SPEC..L.FURY..NL..
                  L.PRIEST..SPEC..L.SHADOW..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.DEVILSAUR,

  DISORIENT =
                  L.HUNTER..NL..
                  L.ROGUE..NL..
                  L.WARLOCK..NL..
                  L.MAGE..SPEC..L.FIRE..NL..
                  L.HUNTER..PET..P.MONKEY,
  SILENCE =
                  L.DEATHKNIGHT..NL..
                  L.MAGE..NL..
                  L.SHAMAN..NL..
                  L.ROGUE..NL..
                  L.WARRIOR..NL..
                  L.DRUID..SPEC..L.FERAL..NL..
                  L.DRUID..SPEC..L.BALANCE..NL..
                  L.HUNTER..SPEC..L.MARKSMAN..NL..
                  L.PALADIN..SPEC..L.PROTECTION..NL..
                  L.PRIEST..SPEC..L.SHADOW..NL..
                  L.WARLOCK..L.PET..P.FELHUNTER..NL..
                  L.HUNTER..L.PET..P.MOTH..NL..
                  L.HUNTER..L.PET..P.NETHERRAY..NL..
                  L.HUNTER..L.PET..P.GORILLA,
  SNARE =
                  L.DEATHKNIGHT..NL..
                  L.HUNTER..NL..
                  L.ROGUE..NL..
                  L.SHAMAN..NL..
                  L.WARLOCK..NL..
                  L.WARRIOR..NL..
                  L.DRUID..SPEC..L.FERAL..NL..
                  L.MAGE..SPEC..L.ARCANE..NL..
                  L.PRIEST..SPEC..L.SHADOW..NL..
                  L.HUNTER..L.PET..P.CROCOLISK..NL..
                  L.HUNTER..L.PET..P.WARPSTALKER..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.CHIMAERA,
  ROOT =
                  L.DRUID..NL..
                  L.MAGE..NL..
                  L.HUNTER..SPEC..L.SURVIVAL..NL..
                  L.SHAMAN..SPEC..L.ELEMENTAL..NL..
                  L.HUNTER..L.PET..P.SPIDER..NL..
                  L.HUNTER..L.PET..P.CRAB..NL..
                  L.HUNTER..L.PET..P.DOG..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.SILITHID,
  KNOCKBACK =
                  L.DRUID..SPEC..L.BALANCE..NL..
                  L.SHAMAN..SPEC..L.ELEMENTAL..NL..
                  L.WARLOCK..L.PET..L.SUCCUBUS..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.RHINO,
  DISARM =
                  L.WARRIOR..NL..
                  L.ROGUE..NL..
                  L.PRIEST..SPEC..L.SHADOW..NL..
                  L.HUNTER..L.PET..P.BIRDOFPREY..NL..
                  L.HUNTER..L.PET..P.SCORPID..NL,
  STUN =
                  L.PALADIN..NL..
                  L.WARRIOR..NL..
                  L.DEATHKNIGHT..SPEC..L.FROST..NL..
                  L.DRUID..SPEC..L.FERAL..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..NL..
                  L.MAGE..SPEC..L.FROST..NL..
                  L.HUNTER..L.PET..P.BAT..NL..
                  L.HUNTER..L.PET..P.WASP..NL..
                  L.HUNTER..SPEC..L.BEASTMASTER..L.PET..P.WORM,
}


-- The button for accessing the checklist
RC.button = CreateFrame ("Button", "RaidChecklistButton", UIParent, "UIPanelButtonTemplate")
RC.button:SetPoint ("CENTER", 0, 0)
RC.button:SetWidth (42)
RC.button:SetHeight (22)
RC.button:SetMovable (true)
RC.button:SetUserPlaced (true)
RC.button:SetClampedToScreen (true)
RC.button:SetScript ("OnMouseDown", function (f) f:StartMoving () end)
RC.button:SetScript ("OnMouseUp", function (f) f:StopMovingOrSizing () end)
RC.button:SetScript ("OnClick", function ()
  if RC.list:IsVisible () then
    RC.list:Hide ()
  else
    RC.list:Show ()
  end
end)

-- The main list frame
RC.list = CreateFrame ("Frame", "RaidChecklistList", RC.button)
local INSET = 5
RC.list:SetBackdrop ({
  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
  edgeSize = 16,
  insets= { left = INSET, right = INSET, top = INSET, bottom = INSET }
})
RC.list.texture = RC.list:CreateTexture (nil, "BACKGROUND")
RC.list.texture:SetTexture (0, 0, 0, 0.75)
RC.list.texture:SetPoint ("TOPLEFT", RC.list, "TOPLEFT", INSET, -INSET)
RC.list.texture:SetPoint ("BOTTOMRIGHT", RC.list, "BOTTOMRIGHT", -INSET, INSET)
RC.list:SetPoint ("TOPLEFT", RC.button, "BOTTOMLEFT", 0, 0)
RC.list:Hide ()

-- A frame to hold the buff entries
RC.bufflist = CreateFrame ("Frame", nil, RC.list)
RC.bufflist:SetPoint ("TOPLEFT", RC.list, "TOPLEFT", INSET, -INSET)
RC.bufflist:SetPoint ("BOTTOM", RC.list, "BOTTOM")

-- A frame to hold the debuff entries
RC.debufflist = CreateFrame ("Frame", nil, RC.list)
RC.debufflist:SetPoint ("TOPLEFT", RC.bufflist, "TOPRIGHT")
RC.debufflist:SetPoint ("BOTTOM", RC.list, "BOTTOM")

-- A frame to hold the CC/PvP entries
RC.pvplist = CreateFrame ("Frame", nil, RC.list)
RC.pvplist:SetPoint ("TOPRIGHT", RC.list, "TOPRIGHT", -INSET, -INSET)
RC.pvplist:SetPoint ("LEFT", RC.debufflist, "RIGHT")
RC.pvplist:SetPoint ("BOTTOM", RC.list, "BOTTOM")

if (RC.colorcodedbackdrop) then
  local texture = RC.bufflist:CreateTexture (nil, "BACKGROUND")
  texture:SetTexture (0, 0, 1, 0.05)
  texture:SetAllPoints ()
  texture = RC.debufflist:CreateTexture (nil, "BACKGROUND")
  texture:SetTexture (1, 0, 0, 0.05)
  texture:SetAllPoints ()
  texture = RC.pvplist:CreateTexture (nil, "BACKGROUND")
  texture:SetTexture (1, 1, 0, 0.05)
  texture:SetAllPoints ()
end

-- The "haven't yet seen talents for" frame
RC.missing = CreateFrame ("Frame", "RaidChecklistMissing", RC.list)
RC.missing:SetBackdrop ({
  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
  edgeSize = 16,
  insets= { left = INSET, right = INSET, top = INSET, bottom = INSET }
})
RC.missing.texture = RC.missing:CreateTexture (nil, "BACKGROUND")
RC.missing.texture:SetTexture (0, 0, 0, 0.75)
RC.missing.texture:SetPoint ("TOPLEFT", RC.missing, "TOPLEFT", INSET, -INSET)
RC.missing.texture:SetPoint ("BOTTOMRIGHT", RC.missing, "BOTTOMRIGHT", -INSET, INSET)
RC.missing:SetPoint ("TOPLEFT", RC.list, "BOTTOMLEFT")
RC.missing:SetPoint ("RIGHT", RC.list, "RIGHT")
RC.missing.hdr = RC.missing:CreateFontString (nil, "ARTWORK", "GameFontNormal")
RC.missing.hdr:SetPoint ("TOPLEFT", RC.missing, "TOPLEFT", INSET, -INSET)
RC.missing.hdr:SetPoint ("RIGHT", RC.missing, "RIGHT", -INSET, 0)
RC.missing.hdr:SetText (L.MISSINGTALENTS)
RC.missing.txt = RC.missing:CreateFontString (nil, "ARTWORK", "GameFontNormal")
RC.missing.txt:SetPoint ("TOPLEFT", RC.missing.hdr, "BOTTOMLEFT", INSET, 0)
RC.missing.txt:SetPoint ("RIGHT", RC.missing.hdr, "RIGHT")
RC.missing.txt:SetWordWrap (true)
RC.missing.txt:SetTextColor (0.7, 0.7, 0.7, 1)
RC.missing:SetScript ("OnEnter", function ()
  GameTooltip:SetOwner (RC.missing, "ANCHOR_RIGHT", 0, -RC.missing:GetHeight ())
  GameTooltip:AddLine (string.gsub (RC.missing_talents_for, ",", "\n"), 0.7, 0.7, 0.7, 1)
  GameTooltip:Show ()
end)
RC.missing:SetScript ("OnLeave", function () GameTooltip:Hide () end)
RC.missing:Hide ()


local function merge_providers ()
  local all = {}
  for k,v in pairs (RC.providers) do
    all[k] = {}
    for guid,_ in pairs (v) do
      all[k][guid] = true
    end
  end
  for k,v in pairs (RC.petproviders) do
    for guid,_ in pairs (v) do
      all[k][guid] = true
    end
  end
  return all
end
RC.merge_providers = merge_providers


local function prepare_tooltip (tag)
  GameTooltip:SetText (L[tag], 1, 1, 1, 1)
  GameTooltip:AddLine (L.AVAILABLEFROM)
  GameTooltip:AddLine (" "..availablefrom[tag])
  GameTooltip:AddLine ("\n"..L.PROVIDEDBY, 1, 0.82, 0, 1)
  local guidlist = merge_providers ()[tag]
  if not next (guidlist) then
    GameTooltip:AddLine (" "..L.NOBODY, 0.7, 0.7, 0.7, 1)
  else
    local names
    for guid,_ in pairs (guidlist) do
      if names then names = names.."\n " else names = " " end
      names = names..RC.provider_names[guid]
    end
    GameTooltip:AddLine (names, 1, 1, 1, 1)
  end
end

local function hide_tooltip ()
  GameTooltip:Hide ()
end


local function make_frame_entry (parent, relto, xoffs, hilight, text, tag)
  local e = CreateFrame ("Frame", nil, parent)
  e:SetPoint ("TOPLEFT", relto, "BOTTOMLEFT")
  local txt = e:CreateFontString (nil, "ARTWORK", "GameFontNormal")
  e:SetWidth (1000) -- Give it enough space to give us a good estimate
  txt:SetPoint ("TOPLEFT", e, "TOPLEFT", xoffs, 0)
  txt:SetPoint ("RIGHT", e, "RIGHT")
  txt:SetJustifyH ("LEFT")
  txt:SetTextColor (0.7, 0.7, 0.7, 1)
  txt:SetText (text)
  e.desired_width = txt:GetStringWidth () * 1.06 + xoffs -- It seems to underestimate :(
  e:SetHeight (txt:GetStringHeight ())
  e:SetPoint ("RIGHT", parent, "RIGHT") -- What we actually want
  e:EnableMouse (true)
  if hilight then
    local tex = e:CreateTexture (nil, "HIGHLIGHT")
    tex:SetAllPoints ()
    tex:SetTexture ("Interface\\Tooltips\\UI-Tooltip-Background") -- FIXME
    tex:SetAlpha (0.3)
    tex:SetVertexColor (1, 1, 0, 0.8)
    e:SetScript ("OnEnter", function ()
      GameTooltip:SetOwner (e, "ANCHOR_RIGHT", 0, -e:GetHeight ())
      prepare_tooltip (e.tag)
      GameTooltip:Show ()
    end)
    e:SetScript ("OnLeave", hide_tooltip)
  end
  e.txt = txt
  e.tag = tag
  return e
end


local function build_lists ()
  RC.buffhdr = make_frame_entry (RC.bufflist, RC.bufflist, 0, false, L.BUFFS)
  RC.buffhdr:SetPoint ("TOPLEFT", RC.bufflist, "TOPLEFT") -- Reposition
  RC.buffhdr.txt:SetTextColor (1, 0.82, 0, 1)
  local totalheight = RC.buffhdr:GetHeight ()

  RC.entries = {}
  local linkto = RC.buffhdr
  local maxwidth = RC.buffhdr.desired_width
  for i,b in ipairs (RC.buffs) do
    local e = make_frame_entry (RC.bufflist, linkto, 8, true, L[b], b)
    maxwidth = max (maxwidth, e.desired_width)
    totalheight = totalheight + e:GetHeight ()
    RC.entries[b] = e
    linkto = e
  end
  RC.bufflist:SetWidth (maxwidth)

  RC.debuffhdr = make_frame_entry (RC.debufflist, RC.buffhdr, 0, false, L.DEBUFFS)
  RC.debuffhdr:SetPoint ("TOPLEFT", RC.buffhdr, "TOPRIGHT") -- Reposition to the right of Buffs list
  RC.debuffhdr.txt:SetTextColor (1, 0.82, 0, 1)
  linkto = RC.debuffhdr
  local maxwidth2 = RC.debuffhdr.desired_width
  for i,b in ipairs (RC.debuffs) do
    local e = make_frame_entry (RC.debufflist, linkto, 8, true, L[b], b)
    maxwidth2 = max (maxwidth2, e.desired_width)
    RC.entries[b] = e
    linkto = e
  end
  RC.debufflist:SetWidth (maxwidth2)

  RC.pvphdr = make_frame_entry (RC.pvplist, RC.debuffhdr, 0, false, L.CCPVP)
  RC.pvphdr:SetPoint ("TOPLEFT", RC.debuffhdr, "TOPRIGHT") -- Reposition to the right of Debuffs list
  RC.pvphdr.txt:SetTextColor (1, 0.82, 0, 1)
  linkto = RC.pvphdr
  local maxwidth3 = RC.pvphdr.desired_width
  for i,b in ipairs (RC.ccpvps) do
    local e = make_frame_entry (RC.pvplist, linkto, 8, true, L[b], b)
    maxwidth3 = max (maxwidth3, e.desired_width)
    RC.entries[b] = e
    linkto = e
  end
  RC.pvplist:SetWidth (maxwidth3)

  RC.list:SetWidth (maxwidth + maxwidth2 +maxwidth3 + 2 * INSET)
  RC.list:SetHeight (totalheight + 3 * INSET) -- Looks better with a bit of extra space
end

build_lists ()

-- Updates the color one each of the buff entries in the list
RC.OnRefresh = function (self)
  local all = merge_providers ()
  for key,guidlist in pairs (all) do
    local c = { r = 0.7, g = 0.7, b = 0.7 }
    if next (guidlist) then
      c = { r = 0, g = 1, b = 0 }
    end
    RC.entries[key].txt:SetTextColor (c.r, c.g, c.b, 1)
  end

  all = nil

  -- Show players we haven't seen talents for yet, if there are any
  if RC.missing_talents_for then
    RC.missing.txt:SetText (string.gsub (RC.missing_talents_for, ",", ", "))
    RC.missing:SetHeight (RC.missing.hdr:GetHeight () * 2 + RC.missing.txt:GetHeight ())
    RC.missing:Show ()
  else
    RC.missing:Hide ()
  end
end

